### Code
#
### author: amelone
###################


### set up

rm(list=ls()) #Removes all items in Environment!

listofpackages <- c("tidyverse", "lubridate", "lmtest", "ggplot2", "stargazer", "tseries", "dynlm", 
                    "sandwich", "tis", "Quandl", "xts", "readxl", 'vars')
for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
} # this routine install packages required if not in library

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))


### import data ####


load('yy.Rdata')
load('MY_US.Rdata')
load('dypot.Rdata')
load('pi_star.Rdata')
load('infl_gap_pi_star.Rdata')
load('y_gap.Rdata')

load('exp_infl_gap_pi_star.Rdata')
load('exp_y_gap.Rdata')
load('y_t_minus_y_t_star.Rdata')

MY <- MY_US
plot(MY)

start_date <- c(1980,1) 

ST_rate <- window(yy[,'y0.25'], start=start_date) 



### TV Policy Rule ####


library(AER)
library(ivpack)

ahead_infl_gap_pi_star <- lag.xts(infl_gap_pi_star, -1)
ahead_y_gap <- lag.xts(y_gap, -1)

df <- na.omit(ts.intersect(ST_rate, MY, dypot, pi_star, 
                                         exp_infl_gap_pi_star, exp_y_gap,
                                         ahead_infl_gap_pi_star, ahead_y_gap, 
                                         infl_gap_pi_star, y_gap))


# IV regression, see Table 1, column 2 in manuscript
TV_reg_IV <- ivreg(ST_rate ~ 0 + MY + dypot + pi_star + ahead_infl_gap_pi_star + ahead_y_gap | 
                    0 + MY + dypot + pi_star + infl_gap_pi_star + y_gap,
                   data = df)

coef_TV_reg_IV <- coef(TV_reg_IV)
coef_TV_reg_IV


# constructing cycle using 2SLS 
fitted_infl <- fitted(lm(ahead_infl_gap_pi_star ~ 0 + MY + dypot + pi_star + infl_gap_pi_star + y_gap,
                    data = df))
fitted_y <- fitted(lm(ahead_y_gap ~ 0 + MY + dypot + pi_star + infl_gap_pi_star + y_gap,
                    data = df))
TV_reg <- lm(ST_rate ~ 0 + MY + dypot + pi_star + fitted_infl + fitted_y,
           data = df)

u_t1 <- ts(resid(TV_reg), start=c(1980,1), frequency = 4)
plot(u_t1)


# AR(1) for cycle
rho <- coef(dynlm(u_t1 ~ 0 + L(u_t1)))



### Construct y_t_*,(40); see equation (3) in paper ####


y_t10 <- window(yy[,'y10'], start=start_date)


# VAR for infl gap and y gap
VAR_data <- ts.intersect(infl_gap_pi_star, y_gap, y_t_minus_y_t_star)

VAR_est <- VAR(y = VAR_data, type= 'none', p = 1)

coef_VAR <- rbind(coef(VAR_est)$infl_gap_pi_star[,1], coef(VAR_est)$y_gap[,1])
coef_VAR


# Equilibrium rate drivers
EH_vars <- ts.intersect(MY, dypot)
EH_vars <- window(EH_vars, start=start_date)

infl_EH <- window(pi_star, start=start_date, end=c(2019,4))
VAR_data_EH <- window(VAR_data, start=start_date, end=c(2019,4))


EH_y_t10 <- c()
for (i in 1:length(ST_rate)) {
  EH_y_t10[i] <- 1/40*(ST_rate[i] + 
                         coef_TV_reg_IV%*%c(EH_vars[i+1,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+1)))+ rho^(1+1)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+2,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+2)))+ rho^(1+2)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+3,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+3)))+ rho^(1+3)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+4,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+4)))+ rho^(1+4)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+5,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+5)))+ rho^(1+5)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+6,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+6)))+ rho^(1+6)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+7,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+7)))+ rho^(1+7)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+8,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+8)))+ rho^(1+8)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+9,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+9)))+ rho^(1+9)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+10,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+10)))+ rho^(1+10)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+11,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+11)))+ rho^(1+11)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+12,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+12)))+ rho^(1+12)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+13,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+13)))+ rho^(1+13)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+14,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+14)))+ rho^(1+14)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+15,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+15)))+ rho^(1+15)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+16,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+16)))+ rho^(1+16)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+17,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+17)))+ rho^(1+17)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+18,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+18)))+ rho^(1+18)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+19,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+19)))+ rho^(1+19)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+20,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+20)))+ rho^(1+20)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+21,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+21)))+ rho^(1+21)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+22,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+22)))+ rho^(1+22)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+23,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+23)))+ rho^(1+23)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+24,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+24)))+ rho^(1+24)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+25,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+25)))+ rho^(1+25)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+26,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+26)))+ rho^(1+26)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+27,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+27)))+ rho^(1+27)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+28,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+28)))+ rho^(1+28)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+29,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+29)))+ rho^(1+29)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+30,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+30)))+ rho^(1+30)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+31,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+31)))+ rho^(1+31)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+32,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+32)))+ rho^(1+32)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+33,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+33)))+ rho^(1+33)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+34,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+34)))+ rho^(1+34)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+35,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+35)))+ rho^(1+35)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+36,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+36)))+ rho^(1+36)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+37,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+37)))+ rho^(1+37)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+38,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+38)))+ rho^(1+38)*u_t1[i] +
                         coef_TV_reg_IV%*%c(EH_vars[i+39,], infl_EH[i], VAR_data_EH[i,]%*%t(coef_VAR^(1+39)))+ rho^(1+39)*u_t1[i])
}
EH_y_t10 <- ts(EH_y_t10, start=start_date, frequency=4)



### Predictive regression, see Table 3, column 4 in manuscript ####


load('rx_annual_y_t10.Rdata')
load('u_t9.Rdata')

yy_US <- na.omit(yy[,3:12])

n <- 10

slope_reg <- yy_US[,(n)] - yy_US[,(1)]
plot(slope_reg)

u_t9_adj <- -(n-1)*u_t9
plot(u_t9_adj)


reg_rx <- dynlm(rx_annual_y_t10 ~ L(slope_reg, 4) + L(u_t9_adj, 4))
stargazer(reg_rx, se=list(coeftest(reg_rx, vcov=NeweyWest)[,2]), type='text')



